package com.sooka.sponest.middleware.base.config;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Arrays;
import java.util.Objects;

/**
 * @Auther: mjq
 * @Date: 2023/7/5 - 07 - 05 - 22:52
 * @Description: com.sooka.sponest.rabbitmq.config
 * @version: 1.0
 */
@Configuration
public class ElasticsearchClientConfig {

    private static final int ADDRESS_LENGTH = 2;

    /**
     * 协议
     */
    @Value("${spring.elasticsearch.scheme:http}")
    private String scheme;

    /**
     * 集群地址，如果有多个用,隔开
     */
    @Value("${spring.elasticsearch.address}")
    private String address;

    /**
     * 获取连接的超时时间
     */
    @Value("${spring.elasticsearch.connectionRequestTimeout}")
    private Integer connectionRequestTimeout;

    /**
     * Socket 连接超时时间
     */
    @Value("${spring.elasticsearch.socketTimeout}")
    private Integer socketTimeout;

    /**
     * 连接超时时间
     */
    @Value("${spring.elasticsearch.connectTimeout}")
    private Integer connectTimeout;

    /**
     * 最大连接数
     */
    @Value("${spring.elasticsearch.maxConnectNum}")
    private Integer maxConnectNum;

    /**
     * 最大路由连接数
     */
    @Value("${spring.elasticsearch.maxConnectPerRoute}")
    private Integer maxConnectPerRoute;

    @Value("${spring.elasticsearch.username}")
    private String username;

    @Value("${spring.elasticsearch.password}")
    private String password;

    @Bean
    public RestClientBuilder restClientBuilder() {
        HttpHost[] hosts = Arrays.stream(address.split(","))
                .map(this::makeHttpHost)
                .filter(Objects::nonNull)
                .toArray(HttpHost[]::new);
        RestClientBuilder restClientBuilder = RestClient.builder(hosts);
        // 异步连接数配置
        restClientBuilder.setHttpClientConfigCallback(httpClientBuilder -> {
            httpClientBuilder.setMaxConnTotal(maxConnectNum);
            httpClientBuilder.setMaxConnPerRoute(maxConnectPerRoute);
            return httpClientBuilder;
        });
        CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(username, password));
        restClientBuilder.setHttpClientConfigCallback(f -> f.setDefaultCredentialsProvider(credentialsProvider));
        // 异步连接延时配置
        restClientBuilder.setRequestConfigCallback(requestConfigBuilder -> {
            requestConfigBuilder.setConnectionRequestTimeout(connectionRequestTimeout);
            requestConfigBuilder.setSocketTimeout(socketTimeout);
            requestConfigBuilder.setConnectTimeout(connectTimeout);
            return requestConfigBuilder;
        });
        return restClientBuilder;
    }

    @Bean(name = "restHighLevelClient")
    public RestHighLevelClient restHighLevelClient(@Autowired RestClientBuilder restClientBuilder) {
        return new RestHighLevelClient(restClientBuilder);
    }

    /**
     * 根据配置创建HttpHost
     *
     * @param s
     * @return
     */
    private HttpHost makeHttpHost(String s) {
        String[] addr = s.split(":");
        if (addr.length == ADDRESS_LENGTH) {
            String ip = addr[0];
            int port = Integer.parseInt(addr[1]);
            return new HttpHost(ip, port, scheme);
        } else {
            return null;
        }
    }
}
