package com.sooka.sponest.middleware.data.visual.controller;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.utils.StringUtils;
import com.sooka.sponest.middleware.base.controller.BaseESController;
import com.sooka.sponest.middleware.data.visual.entity.bo.VisualBO;
import com.sooka.sponest.middleware.data.visual.entity.vo.DeptCountVO;
import com.sooka.sponest.middleware.data.visual.entity.vo.ResourceListVO;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.common.geo.GeoDistance;
import org.elasticsearch.common.unit.DistanceUnit;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

import static com.ruoyi.common.core.constant.DocumentNameConstants.*;
import static com.sooka.sponest.data.api.base.ClassNameConstants.RESOURCE_CLASSMAP;

/**
 * @Field(type=FieldType.Text, analyzer=“ik_max_word”) 表示该字段是一个文本，并作最大程度拆分，默认建立索引
 * @Field(type=FieldType.Text,index=false) 表示该字段是一个文本，不建立索引
 * @Field(type=FieldType.Date) 表示该字段是一个文本，日期类型，默认不建立索引
 * @Field(type=FieldType.Long) 表示该字段是一个长整型，默认建立索引
 * @Field(type=FieldType.Keyword) 表示该字段内容是一个文本并作为一个整体不可分，默认建立索引
 * @Field(type=FieldType.Float) 表示该字段内容是一个浮点类型并作为一个整体不可分，默认建立索引
 * date 、float、long都是不能够被拆分
 */
@RestController
@RequestMapping("/visual")
public class VisualController extends BaseESController {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    static final String LEADER = "leader";

    static final String DEPT_ID = "deptId";

    /**
     * 可视化资源列表
     *
     * @param visualBO
     * @return
     */
    @GetMapping("/getResourceList")
    public R getResourceList(VisualBO visualBO) {
        if (StringUtils.isBlank(visualBO.getType())) {
            return R.fail("getResourceList?type=参数验证异常");
        }
        if (LEADER.equals(visualBO.getAssort()) && StringUtils.isNull(visualBO.getDeptId())) {
            return R.fail("getResourceList?deptId参数验证异常");
        }
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        // 数据权限
        if (LEADER.equals(visualBO.getAssort())) {
            dataPower(queryBuilder, DEPT_ID, visualBO.getDeptId());
        } else {
            dataPower(queryBuilder, DEPT_ID, null);
        }
        List<String> orderRegulation = null;// 指定的排序规则
        switch (visualBO.getType()) {
            case "linye":
                visualBO.setIndexNames(FORESTRY_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(FORESTRY_INDEXNAMES_TZ);
                break;
            case "nongye":
                visualBO.setIndexNames(AGRICULTURE_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(AGRICULTURE_INDEXNAMES_TZ);
                break;
            case "shuili":
                visualBO.setIndexNames(WATER_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(WATER_INDEXNAMES_TZ);
                break;
            case "huanbao":
                visualBO.setIndexNames(ENVIRONMENT_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(ENVIRONMENT_INDEXNAMES_TZ);
                break;
            case "yingji":
                visualBO.setIndexNames(EMERGENCY_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(EMERGENCY_INDEXNAMES_TZ);
                break;
            case "jiaotong":
                visualBO.setIndexNames(TRAFFIC_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(TRAFFIC_INDEXNAMES_TZ);
                break;
            case "ziyuan":
                visualBO.setIndexNames(RESOURCES_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(RESOURCES_INDEXNAMES_TZ);
                break;
            case "xiaofang":
                visualBO.setIndexNames(FIRECONTROL_INDEXNAMES_TZ);
                orderRegulation = Arrays.asList(FIRECONTROL_INDEXNAMES_TZ);
                break;
            default:
                visualBO.setIndexNames(visualBO.getType().split(","));
                orderRegulation = new ArrayList<>();
                break;
        }
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.query(queryBuilder);
        sourceBuilder.aggregation(AggregationBuilders.terms("indexName").field("_index").size(20));
        SearchResponse response = getResponse(visualBO, sourceBuilder);
        // 解析响应
        if (StringUtils.isNotNull(response) && StringUtils.isNotEmpty(orderRegulation)) {
            Terms terms = response.getAggregations().get("indexName");
            List<ResourceListVO> resultList = orderRegulation.stream().map(index -> {
                ResourceListVO vo = new ResourceListVO();
                vo.setType(index);
                vo.setName(RESOURCEMAP_TZ.get(index));
                terms.getBuckets().forEach(item -> {
                    if (index.equals(item.getKeyAsString())) {
                        vo.setNum(item.getDocCount());
                    }
                });
                return vo;
            }).collect(Collectors.toList());
            return R.ok(resultList);
        }
        return R.fail("查询失败");
    }

    /**
     * 可视化资源落点、部门统计
     *
     * @param visualBO
     * @return
     */
    @GetMapping("/getResourcePoint")
    public R getResourcePoint(VisualBO visualBO) {
        if (StringUtils.isBlank(visualBO.getType())) {
            return R.fail("getResourcePoint?type参数验证异常");
        }
        if (LEADER.equals(visualBO.getAssort()) && StringUtils.isNull(visualBO.getDeptId())) {
            return R.fail("getResourcePoint?deptId参数验证异常");
        }
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        if (StringUtils.isNotBlank(visualBO.getId())) {
            queryBuilder.must(QueryBuilders.termQuery("id.keyword", visualBO.getId()));
        }
        if (StringUtils.isNotBlank(visualBO.getName())) {
            queryBuilder.must(QueryBuilders.wildcardQuery("name.keyword", new StringBuilder().append("*").append(visualBO.getName()).append("*").toString()));
        }
        if (StringUtils.isNotNull(visualBO.getDeptId())) {
            dataPower(queryBuilder, DEPT_ID, visualBO.getDeptId());
            /*if ("view".equals(visualBO.getAssort())) {
                queryBuilder.must(QueryBuilders.termQuery(DEPT_ID, visualBO.getDeptId()));
            } else {
            }*/
        }
        // 筛选坐标指定范围内的数据
        if (StringUtils.isNotBlank(visualBO.getLatitude()) && StringUtils.isNotBlank(visualBO.getLongitude())
                && StringUtils.isNotNull(visualBO.getRadius())) {
            queryBuilder.must(QueryBuilders.geoDistanceQuery("location")
                    .point(Double.parseDouble(visualBO.getLatitude()), Double.parseDouble(visualBO.getLongitude()))
                    .distance(visualBO.getRadius(), DistanceUnit.METERS).geoDistance(GeoDistance.PLANE));
            visualBO.setSortFiled("location");
            visualBO.setSortOrder(SortOrder.ASC);
        }
        // 数据权限
        if (!LEADER.equals(visualBO.getAssort())) {
            dataPower(queryBuilder, DEPT_ID, null);
        }
        // 指定返回字段
        visualBO.setIncludeFields(RETURN_FIELDS);
        switch (visualBO.getType()) {
            case "linye":
                visualBO.setIndexNames(FORESTRY_INDEXNAMES_TZ);
                break;
            case "nongye":
                visualBO.setIndexNames(AGRICULTURE_INDEXNAMES_TZ);
                break;
            case "shuili":
                visualBO.setIndexNames(WATER_INDEXNAMES_TZ);
                break;
            case "huanbao":
                visualBO.setIndexNames(ENVIRONMENT_INDEXNAMES_TZ);
                break;
            case "yingji":
                visualBO.setIndexNames(EMERGENCY_INDEXNAMES_TZ);
                break;
            case "jiaotong":
                visualBO.setIndexNames(TRAFFIC_INDEXNAMES_TZ);
                break;
            case "ziyuan":
                visualBO.setIndexNames(RESOURCES_INDEXNAMES_TZ);
                break;
            case "xiaofang":
                visualBO.setIndexNames(FIRECONTROL_INDEXNAMES_TZ);
                break;
            default:
                visualBO.setIndexNames(visualBO.getType().split(","));
                break;
        }
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.query(queryBuilder).size(5000);
        // 查询资源落点
        SearchResponse response = getResponse(visualBO, sourceBuilder);
        List<JSONObject> pointList = getDataList(visualBO, response);
        if (!"view".equals(visualBO.getAssort())) {
            return R.ok(pointList);
        }
        // 部门分组
        sourceBuilder.aggregation(AggregationBuilders.terms(DEPT_ID).field(DEPT_ID).size(500)
                .subAggregation(AggregationBuilders.terms("deptName").field("deptName.keyword")));
        response = getResponse(visualBO, sourceBuilder);
        // 解析响应
        if (StringUtils.isNotNull(response)) {
            Terms terms = response.getAggregations().get(DEPT_ID);
            List<DeptCountVO> deptList = terms.getBuckets().stream().map(item -> {
                DeptCountVO vo = new DeptCountVO();
                vo.setDeptId(item.getKeyAsString());
                Terms names = item.getAggregations().get("deptName");
                vo.setDeptName(names.getBuckets().get(0).getKeyAsString());
                vo.setCount(item.getDocCount());
                return vo;
            }).sorted(Comparator.comparing(DeptCountVO::getCount).reversed()).collect(Collectors.toList());
            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put("pointList", pointList);
            resultMap.put("deptList", deptList);
            return R.ok(resultMap);
        }
        return R.fail("查询失败");
    }

    /**
     * 可视化资源落点、部门统计
     *
     * @param visualBO
     * @return
     */
    @PostMapping("/getResourcePoint")
    public R getResourcePoint2(@RequestBody VisualBO visualBO) {
        return getResourcePoint(visualBO);
    }

    /**
     * 资源详情
     *
     * @param visualBO
     * @return
     */
    @GetMapping("/getResourceDetail")
    public R getResourceDetail(VisualBO visualBO) {
        if (StringUtils.isBlank(visualBO.getType()) || StringUtils.isNull(RESOURCE_CLASSMAP.get(visualBO.getType()))) {
            return R.fail("getResourceDetail?type参数验证异常");
        }
        if (StringUtils.isBlank(visualBO.getId())) {
            return R.fail("getResourceDetail?id参数验证异常");
        }
        visualBO.setIndexNames(visualBO.getType().split(","));
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.query(QueryBuilders.termQuery("id.keyword", visualBO.getId()));
        SearchResponse response = getResponse(visualBO, sourceBuilder);
        List<JSONObject> pointList = getDataList(visualBO, response);
        LinkedHashMap linkedHashMap = null;
        if (StringUtils.isNotEmpty(pointList)) {
            try {
                Object object = JSONObject.parseObject(pointList.get(0).toJSONString(), RESOURCE_CLASSMAP.get(visualBO.getType()));
                ObjectMapper mapper = new ObjectMapper();
                String str = mapper.writeValueAsString(object);
                linkedHashMap = JSONObject.parseObject(str, LinkedHashMap.class);
            } catch (JsonProcessingException e) {
                logger.error(e.getMessage());
            }
        }
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("detail", linkedHashMap);
        resultMap.put("cameras", getCameraList(pointList));
        resultMap.put("pictures", getPictureList(pointList.get(0).getString("id")));
        return R.ok(resultMap);
    }
}
